﻿'HOMEBUTTON STEP 0:  
Public Class HomeNavigationHelper

    Shared Sub New()
        'Subscribe to tombstoning events so we can save the history. 
        AddHandler PhoneApplicationService.Current.Deactivated, AddressOf OnPhoneActivationServiceDeactivated
        AddHandler PhoneApplicationService.Current.Activated, AddressOf OnPhoneApplicationServiceActivated
        AddHandler PhoneApplicationService.Current.Launching, AddressOf OnPhoneApplicationServiceLaunching
    End Sub

#Region "Properties & events"

    Public Shared ReadOnly Property Instance As HomeNavigationHelper
        Get
            If _instance Is Nothing Then _instance = New HomeNavigationHelper
            Return _instance

        End Get
    End Property

    Private _IsInitialized As Boolean
    Public Property IsInitialized As Boolean
        Get
            Return _IsInitialized
        End Get
        Private Set(ByVal value As Boolean)
            _IsInitialized = value
        End Set
    End Property

    Private _IsLoaded As Boolean
    Public Property IsLoaded As Boolean
        Get
            Return _IsLoaded
        End Get
        Private Set(ByVal value As Boolean)
            _IsLoaded = value
        End Set
    End Property

    Public Property AutoJournal As Boolean
        Get
            Return _autoJournal
        End Get
        Set(ByVal value As Boolean)
            If _autoJournal <> value Then
                _autoJournal = value

                If Not AutoJournal Then StopJournaling()
            End If
        End Set
    End Property

    Public Property HideFrameonHomeNav As Boolean
    Public Property ValidateUriBeforeHomeNav As Boolean
    Private _IsDoingHomeNavigation As Boolean
    Public Property IsDoingHomeNavigation As Boolean
        Get
            Return _IsDoingHomeNavigation
        End Get
        Private Set(ByVal value As Boolean)
            _IsDoingHomeNavigation = value
        End Set
    End Property

    Private _IsNavigationInProgress As Boolean
    Public Property IsNavigationInProgress As Boolean
        Get
            Return _IsNavigationInProgress
        End Get
        Private Set(ByVal value As Boolean)
            _IsNavigationInProgress = value
        End Set
    End Property

    Public Event HomeNavigationStarting As EventHandler(Of HomeNavigationStartedEventArgs)
    Public Event HomeNavigationCompleted As EventHandler(Of HomeNavigationCompletedEventArgs)

#End Region

#Region "Public methods"
    Public Function Initialize(ByVal frame As PhoneApplicationFrame,
                               ByVal autoJournal As Boolean,
                               ByVal hideFrameOnNav As Boolean,
                               ByVal validateUriBeforeNav As Boolean) As Boolean
        If Not IsInitialized Then
            AddHandler frame.Navigated, AddressOf frame_Navigated
            AddHandler frame.Navigating, AddressOf frame_Navigating
            AddHandler frame.NavigationStopped, AddressOf frame_NavigationStopped
            _frame = frame
            Me.AutoJournal = autoJournal
            Me.HideFrameonHomeNav = hideFrameOnNav
            Me.ValidateUriBeforeHomeNav = validateUriBeforeNav
            IsInitialized = True

            Return True
        End If
        Return False
    End Function

    'HOMEBUTTON STEP 4:  
    Public Sub SafeNavigate(ByVal targetUri As Uri, Optional ByVal preventCircles As Boolean = False)
        If IsNavigationInProgress Then
            If UriInProgress <> targetUri Then
                _frame.StopLoading()
                System.Diagnostics.Debug.WriteLine("Stopped Loading a nav")
            Else
                System.Diagnostics.Debug.WriteLine("Ignore a double click ")
                Return
            End If
        ElseIf targetUri = _frame.Source Then
            System.Diagnostics.Debug.WriteLine("Ignore fragment navigation")
            Return
        End If

        If preventCircles Then
            If journalingStack.Contains(targetUri) Then
                Me.StartHomeNavigation(targetUri)
            Else
                _frame.Navigate(targetUri)
            End If

        Else
            _frame.Navigate(targetUri)
        End If
    End Sub

    Public Sub StartHomeNavigation(ByVal targetUri As Uri)
        OnStartHomeNavigation(targetUri)
    End Sub

    Public Sub StopHomeNavigation(ByVal completed As Boolean, ByVal userInitiated As Boolean)
        OnStopHomeNavigation(completed, userInitiated)
    End Sub
#End Region

#Region "State"
    Private Const BackNavHistoryKey As String = "BackNavigationHelper.History"
    Private Const HideFrameOnBackNavKey As String = "BackNavigationHelper.HideFrameOnBackNav"
    Private Const AutoJournalKey As String = "BackNavigationHelper.AutoJournal"
    Private Const ValidateUriBeforeBackNavKey As String = "BackNavigationHelper.ValidateUriBeforeBackNav"

    Private Shared Sub OnPhoneApplicationServiceLaunching(ByVal sender As Object, ByVal e As LaunchingEventArgs)
    End Sub

    Private Sub SaveState(ByVal key As String, ByVal value As Object)
        PhoneApplicationService.Current.State(key) = value
    End Sub

    Private Shared Sub OnPhoneActivationServiceDeactivated(ByVal sender As Object, ByVal e As DeactivatedEventArgs)
        Instance.Save()
    End Sub

    Private Sub Save()
        SaveState(BackNavHistoryKey, journalingStack.ToArray())
        SaveState(HideFrameOnBackNavKey, HideFrameonHomeNav)
        SaveState(ValidateUriBeforeBackNavKey, ValidateUriBeforeHomeNav)
        SaveState(AutoJournalKey, AutoJournal)
    End Sub

    Private isActivating As Boolean = False
    Private Sub Load()

        'TODO: Right now we allow duplication of Loading when a chooser that does not 
        'tombstone but does deactivate  
        '             
        System.Diagnostics.Debug.Assert((IsLoaded = True AndAlso PhoneApplicationService.Current.StartupMode = StartupMode.Activate) OrElse IsLoaded = False)

        If PhoneApplicationService.Current.StartupMode = StartupMode.Activate Then

            System.Diagnostics.Debug.WriteLine("Activated")
            Dim historyentries() = LoadState(Of Uri())(BackNavHistoryKey, Nothing)
            If historyentries IsNot Nothing Then
                journalingStack.Clear()
                For x = historyentries.Length - 1 To 0 Step -1
                    journalingStack.Push(historyentries(x))
                Next x
            End If

            Instance.HideFrameonHomeNav = LoadState(Of Boolean)(HideFrameOnBackNavKey, True)
            Instance.AutoJournal = LoadState(Of Boolean)(AutoJournalKey, True)
            Instance.ValidateUriBeforeHomeNav = LoadState(Of Boolean)(ValidateUriBeforeBackNavKey, False)
#If DEBUG Then
        Else
            System.Diagnostics.Debug.WriteLine("Launched")
#End If
        End If
        Me.IsLoaded = True
    End Sub

    Private Function LoadState(Of T)(ByVal key As String, ByVal defaultValue As T) As T
        If PhoneApplicationService.Current.State.ContainsKey(key) Then Return CType(PhoneApplicationService.Current.State(key), T)
        Return defaultValue
    End Function

    Private Property IsTombstoned As Boolean

    Private Shared Sub OnPhoneApplicationServiceActivated(ByVal sender As Object, ByVal e As ActivatedEventArgs)
        Instance.IsTombstoned = True

        'TODO:   This checks if this was truly a tombstone.. 
        ' Some choosers do not tombstone the app in full..  they do deactivate but 
        If PhoneApplicationService.Current.StartupMode = StartupMode.Activate Then Instance.Load()
    End Sub

#End Region


#Region "private and virtual methods"
    Private Sub frame_NavigationStopped(ByVal sender As Object, ByVal e As NavigationEventArgs)
        If IsDoingHomeNavigation Then StopHomeNavigation(False, True)

        IsNavigationInProgress = False
    End Sub

    Private Sub StopJournaling()
        journalingStack.Clear()
    End Sub

    Public Overridable Sub OnStopHomeNavigation(ByVal completed As Boolean, ByVal userInitiated As Boolean)

        Dim eh = Me.HomeNavigationCompletedEvent
        Dim args = New HomeNavigationCompletedEventArgs With
                   {.Destination = _homeNavigationUri,
                    .NavigationCompleted = completed,
                    .UserCancelled = userInitiated}
        If eh IsNot Nothing Then eh(Me, args)

        IsDoingHomeNavigation = False
        _homeNavigationUri = Nothing

        If HideFrameonHomeNav Then _frame.Opacity = frameOpacity
    End Sub

    Private _UriInProgress As Uri
    Public Property UriInProgress As Uri
        Get
            Return _UriInProgress
        End Get
        Friend Set(ByVal value As Uri)
            _UriInProgress = value
        End Set
    End Property

    'HOMEBUTTON STEP 1:  
    Public Overridable Function OnStartHomeNavigation(ByVal targetUri As Uri) As Boolean
        If AutoJournal AndAlso ValidateUriBeforeHomeNav Then
            If Not journalingStack.Contains(targetUri) Then Return False
        End If

        Dim start = Me.HomeNavigationStartingEvent
        Dim args = New HomeNavigationStartedEventArgs With {.Destination = targetUri}
        If start IsNot Nothing Then start(Me, args)

        'if no page objected to a back nav then do it. 
        If Not args.Cancel Then
            IsDoingHomeNavigation = True
            _homeNavigationUri = targetUri

            If HideFrameonHomeNav Then
                frameOpacity = _frame.Opacity
                _frame.Opacity = 0
            End If

            _frame.GoBack()
        End If
        Return Not args.Cancel
    End Function

    Private Sub frame_Navigating(ByVal sender As Object, ByVal e As System.Windows.Navigation.NavigatingCancelEventArgs)
        currentDirection = e.NavigationMode
        UriInProgress = e.Uri
#If DEBUG Then
        Select Case e.NavigationMode
            Case NavigationMode.Back
            Case NavigationMode.New
            Case NavigationMode.Refresh
                System.Diagnostics.Debug.Assert(False, "trying to learn when this happens")
            Case NavigationMode.Forward
                System.Diagnostics.Debug.Assert(False, "trying to learn when this happens")
        End Select
#End If
        Me.IsNavigationInProgress = True
    End Sub


    'HOMEBUTTON STEP 2:  
    Private Sub frame_Navigated(ByVal sender As Object,
                                ByVal e As System.Windows.Navigation.NavigationEventArgs)
        'HOMEBUTTON STEP 2.1:   Autojournaling is adding stuff to the stack so we can validate later that a URL is in the stack
        If AutoJournal Then
            If currentDirection = NavigationMode.Back Then
                If journalingStack.Count > 0 Then

                    If Not IsTombstoned Then
                        journalingStack.Pop()
                    Else
                        IsTombstoned = False
                    End If
#If DEBUG Then
                    If journalingStack.Count > 0 Then
                        Dim top = journalingStack.Peek()
                        System.Diagnostics.Debug.Assert(top Is e.Uri)
                    End If
#End If
                    'we have reached pages that were not on our stack. This likely won't happen 
                    'for real apps, but happens for demos when you turn journaling on/off.. 
                Else
                    journalingStack.Push(e.Uri)
                End If
            End If

            If currentDirection = NavigationMode.New Then
                'SKIP the tombstoning URLs 
                If e.Uri Is New Uri("app://external", UriKind.RelativeOrAbsolute) Then
                    IsTombstoned = True
                    System.Diagnostics.Debug.WriteLine("*** External *** ")
                Else

                    journalingStack.Push(e.Uri)
                End If
            End If
        End If

        'HOMEBUTTON STEP 2.2:  Check for back navigation and continue the sequence
        If IsDoingHomeNavigation Then
            If e.Uri IsNot _homeNavigationUri AndAlso _frame.CanGoBack Then
                _frame.GoBack()
                Return
            Else
                StopHomeNavigation(True, False)
            End If

        End If
        IsNavigationInProgress = False
    End Sub

#End Region

#Region "members"
    Private _frame As PhoneApplicationFrame
    Private _homeNavigationUri As Uri
    Private Shared _instance As HomeNavigationHelper
    Private _autoJournal As Boolean
    Private journalingStack As New System.Collections.Generic.Stack(Of Uri)
    Public Shared DemoSettings As New Settings
    Private currentDirection As NavigationMode
    Private frameOpacity As Double

    Public Class Settings
        Public Property HideAppBarOnNavigate As Boolean
        Public Property PreventLoopCreation As Boolean
    End Class
#End Region
End Class

Public Class HomeNavigationStartedEventArgs
    Inherits EventArgs
    Public Destination As Uri
    Public Cancel As Boolean
End Class

Public Class HomeNavigationCompletedEventArgs
    Inherits EventArgs
    Public Destination As Uri
    Public NavigationCompleted As Boolean
    Public UserCancelled As Boolean
End Class
