﻿Partial Public Class ItemPage
    Inherits PhoneApplicationPage

    Private navigationCompletedListener As WeakEventListener(Of ItemPage, Object, HomeNavigationCompletedEventArgs)
    Private navigationStartedListener As WeakEventListener(Of ItemPage, Object, HomeNavigationStartedEventArgs)

    Public Sub New()
        InitializeComponent()

#If DEBUG Then
        instanceCount += 1
#End If
    End Sub

#If DEBUG Then
    Private Shared instanceCount As Integer
    Protected Overrides Sub Finalize()
        System.Diagnostics.Debug.WriteLine("~ItemPage: {0} instances left", instanceCount)
        instanceCount -= 1
    End Sub

#End If

    Private appBarVisibility As Boolean
    Private Sub Instance_HomeNavigationCompleted(ByVal sender As Object, ByVal e As HomeNavigationCompletedEventArgs)
        If HomeNavigationHelper.DemoSettings.HideAppBarOnNavigate Then Me.ApplicationBar.IsVisible = appBarVisibility
    End Sub

    Private Sub Instance_BackNavigationStarting(ByVal sender As Object, ByVal e As HomeNavigationStartedEventArgs)
        If HomeNavigationHelper.DemoSettings.HideAppBarOnNavigate Then
            appBarVisibility = Me.ApplicationBar.IsVisible
            Me.ApplicationBar.IsVisible = False
        End If
    End Sub

    Public Shared [Next] As Integer
    Public instanceId As Integer
    Private level As Integer = 0
    Private item As Integer = 0
    Protected Overrides Sub OnNavigatedTo(ByVal e As System.Windows.Navigation.NavigationEventArgs)
        If Not HomeNavigationHelper.Instance.IsDoingHomeNavigation Then
            MyBase.OnNavigatedTo(e)
            level = Integer.Parse(NavigationContext.QueryString("level"))
            If NavigationContext.QueryString.ContainsKey("item") Then item = Integer.Parse(NavigationContext.QueryString("item"))

            Me.PageTitle.Text = "level " & level & ",item=" & item

            If level > 0 Then upLevel.IsEnabled = True

            Dim appbarstring As String = Nothing

            If NavigationContext.QueryString.TryGetValue("appbar", appbarstring) Then
                Dim showAppbarFlag As Boolean
                Boolean.TryParse(appbarstring, showAppbarFlag)
                Me.ApplicationBar.IsVisible = showAppbarFlag
                Me.showAppBar.IsChecked = showAppbarFlag

                '                     
                '*  If we are showing applicationbar and are not listening for BackNavigationHelper's navigation events, we should subscribe
                '*  SAMPLE NOTE:  
                '*  This makes huge assumptions that Appbar will not be enabled via page interactions 
                '*  I did it that way to demonstrate that the navigation events are pay per play.. but I assume most apps will subscribe to
                '*  this all the time 

                'HOMEBUTTON STEP 3:  
                If showAppbarFlag AndAlso navigationCompletedListener Is Nothing Then
                    navigationCompletedListener = New WeakEventListener(Of ItemPage, Object, HomeNavigationCompletedEventArgs)(Me)
                    navigationCompletedListener.OnEventAction =
                        Sub(instance, source, args) instance.Instance_HomeNavigationCompleted(source, args)
                    navigationCompletedListener.OnDetachAction =
                        Sub(listener)
                            RemoveHandler HomeNavigationHelper.Instance.HomeNavigationCompleted, AddressOf listener.OnEvent
                        End Sub

                    AddHandler HomeNavigationHelper.Instance.HomeNavigationCompleted, AddressOf navigationCompletedListener.OnEvent


                    navigationStartedListener = New WeakEventListener(Of ItemPage, Object, HomeNavigationStartedEventArgs)(Me)
                    navigationStartedListener.OnEventAction = Sub(instance, source, args) instance.Instance_BackNavigationStarting(source, args)
                    navigationStartedListener.OnDetachAction =
                        Sub(listener)
                            RemoveHandler HomeNavigationHelper.Instance.HomeNavigationStarting, AddressOf listener.OnEvent
                        End Sub

                    AddHandler HomeNavigationHelper.Instance.HomeNavigationStarting, AddressOf navigationStartedListener.OnEvent
                End If

                '                     
                '* This just detaches the event when the applicationbar is not shown, since we then would not need callbacks
                '                     
                If (Not showAppbarFlag) AndAlso navigationCompletedListener IsNot Nothing Then
                    navigationCompletedListener.Detach()
                    navigationCompletedListener = Nothing

                    navigationStartedListener.Detach()
                    navigationStartedListener = Nothing
                End If
            End If
        Else
            If Me.ApplicationBar.IsVisible Then
                System.Diagnostics.Debug.WriteLine("Hiding appbar in tombstone back")
                appBarVisibility = Me.ApplicationBar.IsVisible
                Me.ApplicationBar.IsVisible = HomeNavigationHelper.DemoSettings.HideAppBarOnNavigate
            End If
        End If
    End Sub

    Private Sub GoHome(ByVal sender As Object, ByVal args As RoutedEventArgs)
        HomeNavigationHelper.Instance.StartHomeNavigation(New Uri("/HomeNavigationSamplePages/LevelPage.xaml?level=0", UriKind.Relative))
    End Sub

    Private Sub GoUp(ByVal sender As Object, ByVal args As RoutedEventArgs)
        Dim targetUri As New Uri(String.Format("/HomeNavigationSamplePages/LevelPage.xaml?level={0}&appbar={1}",
                                               level - 1,
                                               showAppBar.IsChecked.Value),
                                           UriKind.Relative)

        If HomeNavigationHelper.DemoSettings.PreventLoopCreation Then
            HomeNavigationHelper.Instance.SafeNavigate(targetUri, True)
        Else
            HomeNavigationHelper.Instance.StartHomeNavigation(targetUri)
        End If
    End Sub

    Private Sub GoDown(ByVal u As Object, ByVal u2 As RoutedEventArgs)

        NavigationService.Navigate(New Uri(String.Format("/HomeNavigationSamplePages/LevelPage.xaml?level={0}&appbar={1}", level + 1, showAppBar.IsChecked.Value), UriKind.Relative))
    End Sub

    Private Sub GoBack(ByVal u As Object, ByVal u2 As RoutedEventArgs)
        If NavigationService.CanGoBack Then NavigationService.GoBack()
    End Sub

    Private Sub GoToItem(ByVal u As Object, ByVal u2 As RoutedEventArgs)
        Dim targetUri As New Uri(String.Format("/HomeNavigationSamplePages/ItemPage.xaml?level={0}&item={1}&appbar={2}", level, item + 1, showAppBar.IsChecked.Value), UriKind.Relative)

        HomeNavigationHelper.Instance.SafeNavigate(targetUri)
    End Sub


    Private Sub OnLaunchChooser(ByVal u As Object, ByVal u2 As RoutedEventArgs)
        Dim chooser As New Microsoft.Phone.Tasks.PhotoChooserTask
        chooser.ShowCamera = True
        chooser.Show()
    End Sub
End Class